@@ -26,6 +26,14 @@
 #include <string.h>
 #include <ctype.h>
 #endif /* STDC_HEADERS */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+#if HAVE_FCNTL_H
+#include <fcntl.h>
+#endif /* HAVE_FCNTL_H */
+#include <sys/types.h>
+#include <sys/stat.h>
 #if TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
@@ -51,6 +59,7 @@
 #include "tool-hostrange-common.h"
 #include "tool-sdr-cache-common.h"
 #include "tool-util-common.h"
+#include "fd.h"
 
 typedef int (*Bmc_device_system_info_first_set)(ipmi_ctx_t ctx,
 						uint8_t set_selector,
@@ -2721,6 +2730,301 @@ set_base_os_hypervisor_url (bmc_device_state_data_t *state_data)
 }
 
 
+static int
+read_fru (bmc_device_state_data_t *state_data)
+{
+  uint8_t areabuf[IPMI_FRU_AREA_SIZE_MAX+1];
+  unsigned int area_type = 0;
+  unsigned int area_length = 0;
+  unsigned int flags = 0;
+  int fd = -1;
+  int rv = -1;
+
+  assert (state_data);
+
+  if (!(state_data->fru_ctx = ipmi_fru_ctx_create (state_data->ipmi_ctx)))
+    {
+      pstdout_perror (state_data->pstate, "ipmi_fru_ctx_create()");
+      goto cleanup;
+    }
+  
+  if (state_data->prog_data->args->common_args.debug)
+    flags |= IPMI_FRU_FLAGS_DEBUG_DUMP;
+
+  /* Don't care about checksum checks now */
+  flags |= IPMI_FRU_FLAGS_SKIP_CHECKSUM_CHECKS;
+  flags |= IPMI_FRU_FLAGS_READ_RAW;
+
+  if (ipmi_fru_ctx_set_flags (state_data->fru_ctx, flags) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "ipmi_fru_ctx_set_flags: %s\n",
+		       ipmi_fru_ctx_strerror (ipmi_fru_ctx_errnum (state_data->fru_ctx)));
+      goto cleanup;
+    }
+  
+  if (ipmi_fru_open_device_id (state_data->fru_ctx, state_data->prog_data->args->device_id) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+                       stderr,
+                       "ipmi_fru_open_device_id: %s\n",
+                       ipmi_fru_ctx_errormsg (state_data->fru_ctx));
+      goto cleanup;
+    }
+
+  memset (areabuf, '\0', IPMI_FRU_AREA_SIZE_MAX + 1);
+  if (ipmi_fru_read_data_area (state_data->fru_ctx,
+			       &area_type,
+			       &area_length,
+			       areabuf,
+			       IPMI_FRU_AREA_SIZE_MAX) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "ipmi_fru_read_data_area: %s\n",
+		       ipmi_fru_ctx_errormsg (state_data->fru_ctx));
+      goto cleanup;
+    }
+
+  if (area_type != IPMI_FRU_AREA_TYPE_RAW_DATA)
+    {
+      pstdout_printf (state_data->pstate,
+                      "FRU Error: Invalid area type returned\n");
+      goto out;
+    }
+
+  if (area_length)
+    {
+      if ((fd = open (state_data->prog_data->args->read_fru_filename, O_CREAT | O_WRONLY, 0600)) < 0)
+	{
+	  pstdout_fprintf (state_data->pstate,
+			   stderr,
+			   "Cannot open '%s': %s\n",
+			   state_data->prog_data->args->read_fru_filename,
+			   strerror (errno));
+	  goto out;
+	}
+
+      if (fd_write_n (fd, areabuf, area_length) < 0)
+	{
+	  pstdout_fprintf (state_data->pstate,
+			   stderr,
+			   "fd_write_n: %s\n",
+			   strerror (errno));
+	  goto out;
+	}
+    }
+  else
+    {
+      pstdout_printf (state_data->pstate,
+                      "FRU Error: FRU area is zero length\n");
+      goto out;
+    }
+
+  
+ /* Close of fru_ctx in run_cmd_args */
+ out:
+  rv = 0;
+ cleanup:
+  ipmi_fru_close_device_id (state_data->fru_ctx);
+  close (fd);
+  return (rv);
+}
+
+static int
+write_fru (bmc_device_state_data_t *state_data)
+{
+  uint8_t areabuf[IPMI_FRU_AREA_SIZE_MAX+1];
+  fiid_obj_t fru_get_inventory_rs = NULL;
+  fiid_obj_t obj_cmd_rs = NULL;
+  struct stat sbuf;
+  unsigned int area_size;
+  unsigned int area_offset = 0;
+  unsigned int blocksize = 16;
+  ssize_t len;
+  int fd = -1;
+  int rv = -1;
+  int loop_errors = 0;
+  int loop_errors_max = 5;	/* arbitrarily selected */
+  unsigned int percent = 0;
+  uint64_t val;
+
+  assert (state_data);
+
+  if (stat (state_data->prog_data->args->write_fru_filename, &sbuf) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "Cannot stat '%s': %s\n",
+		       state_data->prog_data->args->write_fru_filename,
+		       strerror (errno));
+      goto cleanup;
+    }
+
+  if (!sbuf.st_size)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "FRU file '%s' is empty\n",
+		       state_data->prog_data->args->write_fru_filename);
+      goto cleanup;
+    }
+
+  if (!(fru_get_inventory_rs = fiid_obj_create (tmpl_cmd_get_fru_inventory_area_info_rs)))
+    {
+      pstdout_fprintf (state_data->pstate,
+                       stderr,
+                       "fiid_obj_create: %s\n",
+                       strerror (errno));
+      goto cleanup;
+    }
+
+  if (ipmi_cmd_get_fru_inventory_area_info (state_data->ipmi_ctx,
+					    state_data->prog_data->args->device_id,
+					    fru_get_inventory_rs) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+                       stderr,
+                       "ipmi_cmd_get_fru_inventory_area_info: %s\n",
+                       ipmi_ctx_errormsg (state_data->ipmi_ctx));
+      goto cleanup;
+    }
+
+  if (FIID_OBJ_GET (fru_get_inventory_rs,
+		    "fru_inventory_area_size",
+		    &val) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+                       stderr,
+                       "fiid_obj_get: 'fru_inventory_area_size': %s\n",
+                       fiid_obj_errormsg (fru_get_inventory_rs));
+      goto cleanup;
+    }
+  area_size = val;
+
+  if (area_size != sbuf.st_size)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "FRU file '%s':  size (%u) does not match area size (%u)\n",
+		       sbuf.st_size,
+		       area_size);
+      goto cleanup;
+    }
+
+  memset (areabuf, '\0', IPMI_FRU_AREA_SIZE_MAX + 1);
+  if ((fd = open (state_data->prog_data->args->write_fru_filename, O_RDONLY)) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "Cannot open '%s': %s\n",
+		       state_data->prog_data->args->write_fru_filename,
+		       strerror (errno));
+      goto cleanup;
+    }
+  
+  if ((len = fd_read_n (fd, areabuf, area_size)) < 0)
+    {
+      pstdout_fprintf (state_data->pstate,
+		       stderr,
+		       "fd_read_n: %s\n",
+		       strerror (errno));
+      goto cleanup;
+    }
+
+  if (!(obj_cmd_rs = fiid_obj_create (tmpl_cmd_write_fru_data_rs)))
+    {
+      pstdout_fprintf (state_data->pstate,
+                       stderr,
+                       "fiid_obj_create: %s\n",
+                       strerror (errno));
+      goto cleanup;
+    }
+
+  if (state_data->prog_data->args->verbose)
+    fprintf (stderr, "%u%%\r", percent);
+
+  while (area_offset < area_size)
+    {
+      unsigned int count_written;
+
+      if ((area_size - area_offset) < blocksize)
+	blocksize = area_size - area_offset;
+
+      if (ipmi_cmd_write_fru_data (state_data->ipmi_ctx,
+                                   state_data->prog_data->args->device_id,
+                                   area_offset,
+                                   areabuf + area_offset,
+                                   blocksize,
+                                   obj_cmd_rs) < 0)
+	{
+	  if (ipmi_ctx_errnum (state_data->ipmi_ctx) == IPMI_ERR_BAD_COMPLETION_CODE)
+	    {
+	      /* Assume blocksize is too large */
+	      if (ipmi_check_completion_code (obj_cmd_rs, IPMI_COMP_CODE_CANNOT_RETURN_REQUESTED_NUMBER_OF_BYTES) == 1)
+		{
+		  if (!(blocksize <= 8))
+		    blocksize /= 2;
+		  else
+		    goto error_out;
+		}
+	      
+	      if (ipmi_check_completion_code (obj_cmd_rs, IPMI_COMP_CODE_WRITE_FRU_DATA_FRU_DEVICE_BUSY) == 1)
+		{
+		  loop_errors++;
+
+		  if (loop_errors > loop_errors_max)
+		    goto error_out;
+		}
+
+	      continue;
+	    }
+
+	error_out:
+	  pstdout_fprintf (state_data->pstate,
+			   stderr,
+			   "ipmi_cmd_write_fru_data: %s\n",
+			   ipmi_ctx_errormsg (state_data->ipmi_ctx));
+	  goto cleanup;
+	}
+
+      if (FIID_OBJ_GET (obj_cmd_rs,
+			"count_written",
+			&val) < 0)
+	{
+	  pstdout_fprintf (state_data->pstate,
+			   stderr,
+			   "fiid_obj_get: 'count_written': %s\n",
+			   fiid_obj_errormsg (fru_get_inventory_rs));
+	  goto cleanup;
+	}
+      count_written = val;
+
+      area_offset += count_written;
+
+      if (state_data->prog_data->args->verbose)
+	{
+	  unsigned int newpercent = (unsigned int)(((double)area_offset/area_size) * 100);
+	  if (newpercent > percent)
+	    {
+	      percent = newpercent;
+	      fprintf (stderr, "%u%%\r", percent);
+	    }
+	}
+    }
+
+  if (state_data->prog_data->args->verbose)
+    fprintf (stderr, "100%%\r\n");
+
+  rv = 0;
+ cleanup:
+  fiid_obj_destroy (fru_get_inventory_rs);
+  fiid_obj_destroy (obj_cmd_rs);
+  close (fd);
+  return (rv);
+}
+
 static int
 run_cmd_args (bmc_device_state_data_t *state_data)
 {
@@ -2817,6 +3121,12 @@ run_cmd_args (bmc_device_state_data_t *state_data)
   if (args->set_base_os_hypervisor_url)
     return (set_base_os_hypervisor_url (state_data));
 
+  if (args->read_fru)
+    return (read_fru (state_data));
+  
+  if (args->write_fru)
+    return (write_fru (state_data));
+
   rv = 0;
   return (rv);
 }
@@ -2843,7 +3153,7 @@ _bmc_device (pstdout_state_t pstate,
 	return (EXIT_FAILURE);
       return (EXIT_SUCCESS);
     }
-  
+
   memset (&state_data, '\0', sizeof (bmc_device_state_data_t));
   state_data.prog_data = prog_data;
   state_data.pstate = pstate;
@@ -2867,6 +3177,7 @@ _bmc_device (pstdout_state_t pstate,
   exit_code = EXIT_SUCCESS;
  cleanup:
   ipmi_sdr_ctx_destroy (state_data.sdr_ctx);
+  ipmi_fru_ctx_destroy (state_data.fru_ctx);
   ipmi_ctx_close (state_data.ipmi_ctx);
   ipmi_ctx_destroy (state_data.ipmi_ctx);
   return (exit_code);
@@ -2894,6 +3205,21 @@ main (int argc, char **argv)
   if (!hosts_count)
     return (EXIT_SUCCESS);
 
+  /* Handle special case exceptions */
+  if (prog_data.args->read_fru && hosts_count > 1)
+    {
+      fprintf (stderr,
+	       "Cannot execute --read-fru on multiple hosts\n");
+      return (EXIT_FAILURE);
+    }
+
+  if (prog_data.args->write_fru && hosts_count > 1 && prog_data.args->verbose > 0)
+    {
+      fprintf (stderr,
+	       "Cannot execute --write-fru w/ verbosity on multiple hosts, setting verbosity to 0\n");
+      prog_data.args->verbose = 0;
+    }
+
   /* We don't want caching info to output when are doing ranged output */
   if (hosts_count > 1)
     prog_data.args->common_args.quiet_cache = 1;
