@@ -34,6 +34,7 @@
 #include "ipmi-oem-common.h"
 #include "ipmi-oem-dell.h"
 #include "ipmi-oem-fujitsu.h"
+#include "ipmi-oem-gigabyte.h"
 #include "ipmi-oem-ibm.h"
 #include "ipmi-oem-intel.h"
 #include "ipmi-oem-intelnm.h"
@@ -502,6 +503,31 @@ struct ipmi_oem_command oem_fujitsu[] =
     },
   };
 
+struct ipmi_oem_command oem_gigabyte[] =
+  {
+    {
+      "get-nic-mode",
+      NULL,
+      0,
+      IPMI_OEM_COMMAND_FLAGS_DEFAULT,
+      ipmi_oem_gigabyte_get_nic_mode
+    },
+    {
+      "set-nic-mode",
+      "<dedicated|shared|failover>",
+      1,
+      IPMI_OEM_COMMAND_FLAGS_DEFAULT,
+      ipmi_oem_gigabyte_set_nic_mode
+    },
+    {
+      NULL,
+      NULL,
+      0,
+      IPMI_OEM_COMMAND_FLAGS_DEFAULT,
+      NULL
+    },
+  };
+
 struct ipmi_oem_command oem_ibm[] =
   {
     {
@@ -550,22 +576,20 @@ struct ipmi_oem_command oem_intel[] =
       IPMI_OEM_COMMAND_FLAGS_DEFAULT,
       ipmi_oem_intel_set_power_restore_delay
     },
-#if 0
     {
-      "get-bmc-service-status",
+      "get-bmc-services",
       NULL,
       0,
       IPMI_OEM_COMMAND_FLAGS_DEFAULT,
-      ipmi_oem_intel_get_bmc_service_status
+      ipmi_oem_intel_get_bmc_services
     },
     {
-      "set-bmc-service-status",
-      "<enable|disable> <ssh|http|kvm>",
+      "set-bmc-services",
+      "<enable|disable> <ssh|http|rmcp|kvm>",
       2,
       IPMI_OEM_COMMAND_FLAGS_DEFAULT,
-      ipmi_oem_intel_set_bmc_service_status
+      ipmi_oem_intel_set_bmc_services
     },
-#endif
     {
       "restore-configuration",
       NULL,
@@ -1590,6 +1614,10 @@ struct ipmi_oem_id oem_cb[] =
       "IBM",
       oem_ibm
     },
+    {
+      "GIGABYTE",
+      oem_gigabyte
+    },
     {
       "Intel",
       oem_intel
