@@ -679,6 +679,8 @@ ipmi_ssif_ctx_set_flags (ipmi_ssif_ctx_t ctx, unsigned int flags)
 int
 ipmi_ssif_ctx_io_init (ipmi_ssif_ctx_t ctx)
 {
+  int flags;
+
   if (!ctx || ctx->magic != IPMI_SSIF_CTX_MAGIC)
     {
       ERR_TRACE (ipmi_ssif_ctx_errormsg (ctx), ipmi_ssif_ctx_errnum (ctx));
@@ -701,6 +703,19 @@ ipmi_ssif_ctx_io_init (ipmi_ssif_ctx_t ctx)
       goto cleanup;
     }
 
+  flags = fcntl(ctx->device_fd, F_GETFD);
+  if (flags < 0)
+    {
+      SSIF_SET_ERRNUM (ctx, errno);
+      goto cleanup;
+    }
+  flags |= FD_CLOEXEC;
+  if (fcntl(ctx->device_fd, F_SETFD, flags) < 0)
+    {
+      SSIF_SET_ERRNUM (ctx, errno);
+      goto cleanup;
+    }
+
   if (ioctl (ctx->device_fd,
              IPMI_I2C_SLAVE,
              ctx->driver_address) < 0)
