@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2014 FreeIPMI Core Team
+ * Copyright (C) 2003-2015 FreeIPMI Core Team
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,41 +26,22 @@
 #include <string.h>
 #include <stdarg.h>
 #endif /* STDC_HEADERS */
-#if TIME_WITH_SYS_TIME
-#include <sys/time.h>
-#include <time.h>
-#else /* !TIME_WITH_SYS_TIME */
-#if HAVE_SYS_TIME_H
-#include <sys/time.h>
-#else /* !HAVE_SYS_TIME_H */
-#include <time.h>
-#endif /* !HAVE_SYS_TIME_H */
-#endif /* !TIME_WITH_SYS_TIME */
 #include <assert.h>
 #include <errno.h>
 
 #include "freeipmi/sel/ipmi-sel.h"
 
-#include "freeipmi/cmds/ipmi-sel-cmds.h"
 #include "freeipmi/cmds/ipmi-oem-intel-node-manager-cmds.h"
+#include "freeipmi/sdr/oem/ipmi-sdr-oem-intel-node-manager.h"
 #include "freeipmi/record-format/ipmi-sdr-record-format.h"
-#include "freeipmi/record-format/ipmi-sdr-oem-record-format.h"
 #include "freeipmi/record-format/ipmi-sel-record-format.h"
-#include "freeipmi/sdr/ipmi-sdr-oem.h"
 #include "freeipmi/spec/ipmi-event-reading-type-code-spec.h"
-#include "freeipmi/spec/ipmi-event-reading-type-code-oem-spec.h"
 #include "freeipmi/spec/ipmi-iana-enterprise-numbers-spec.h"
 #include "freeipmi/spec/ipmi-product-id-spec.h"
-#include "freeipmi/spec/ipmi-sensor-and-event-code-tables-spec.h"
-#include "freeipmi/spec/ipmi-sensor-and-event-code-tables-oem-spec.h"
-#include "freeipmi/spec/ipmi-sensor-numbers-oem-spec.h"
-#include "freeipmi/spec/ipmi-sensor-types-spec.h"
-#include "freeipmi/spec/ipmi-sensor-types-oem-spec.h"
-#include "freeipmi/spec/ipmi-sensor-units-spec.h"
-#include "freeipmi/spec/ipmi-slave-address-spec.h"
-#include "freeipmi/spec/ipmi-slave-address-oem-spec.h"
-#include "freeipmi/util/ipmi-iana-enterprise-numbers-util.h"
-#include "freeipmi/util/ipmi-sensor-and-event-code-tables-util.h"
+#include "freeipmi/spec/oem/ipmi-event-reading-type-code-oem-intel-node-manager-spec.h"
+#include "freeipmi/spec/oem/ipmi-sensor-and-event-code-tables-oem-intel-node-manager-spec.h"
+#include "freeipmi/spec/oem/ipmi-sensor-numbers-oem-intel-node-manager-spec.h"
+#include "freeipmi/spec/oem/ipmi-sensor-types-oem-intel-node-manager-spec.h"
 
 #include "ipmi-sel-common.h"
 #include "ipmi-sel-defs.h"
@@ -78,13 +59,13 @@
  *
  * Intel S5500WB/Penguin Computing Relion 700
  * Intel S2600JF/Appro 512X
+ * Intel S2600KP
+ * Intel S2600WT2
+ * Intel S2600WTT
  * Inventec 5441/Dell Xanadu II
  * Inventec 5442/Dell Xanadu III
  * Quanta S99Q/Dell FS12-TY
  * Quanta QSSC-S4R/Appro GB812X-CN
- * Intel S2600KP
- * Intel S2600WT2
- * Intel S2600WTT
  *
  * Should be called from ipmi-sel-string-VENDOR.c files, not
  * from ipmi-sel-string.c.
@@ -476,6 +457,15 @@ sel_string_output_intel_node_manager_event_data1_class_oem (ipmi_sel_ctx_t ctx,
 
           power_limiting_capability = system_event_record_data->offset_from_event_reading_type_code & IPMI_OEM_INTEL_NODE_MANAGER_OPERATIONAL_CAPABILITIES_CHANGE_EVENT_EVENT_DATA1_POWER_LIMITING_CAPABILITY_BITMASK;
           power_limiting_capability >>= IPMI_OEM_INTEL_NODE_MANAGER_OPERATIONAL_CAPABILITIES_CHANGE_EVENT_EVENT_DATA1_POWER_LIMITING_CAPABILITY_SHIFT;
+
+          snprintf (tmpbuf,
+                    tmpbuflen,
+                    "Policy Interface = %s, Monitoring = %s, Power Limiting = %s",
+		    policy_interface_capability ? "Available" : "Not Available",
+		    monitoring_capability ? "Available" : "Not Available",
+		    power_limiting_capability ? "Available" : "Not Available");
+	  
+          return (1);
         }
 
       if (system_event_record_data->event_type_code == IPMI_EVENT_READING_TYPE_CODE_OEM_INTEL_NODE_MANAGER_ALERT_THRESHOLD_EXCEEDED
@@ -519,6 +509,31 @@ sel_string_output_intel_node_manager_event_data1_class_oem (ipmi_sel_ctx_t ctx,
 	  
           return (1);
         }
+
+      if (system_event_record_data->event_type_code == IPMI_EVENT_READING_TYPE_CODE_OEM_INTEL_NODE_MANAGER_CUPS_EVENT
+	  && system_event_record_data->sensor_number == IPMI_SENSOR_NUMBER_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_SENSOR)
+        {
+	  uint8_t cups_event;
+
+          cups_event = system_event_record_data->offset_from_event_reading_type_code & IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA1_CUPS_POLICY_EVENT_BITMASK;
+          cups_event >>= IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA1_CUPS_POLICY_EVENT_SHIFT;
+
+          if (cups_event == IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA1_CUPS_EVENT_EVENT_THRESHOLD_EXCEEDED)
+            {
+              uint8_t threshold_number;
+
+              threshold_number = system_event_record_data->offset_from_event_reading_type_code & IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA1_THRESHOLD_NUMBER_BITMASK;
+              threshold_number >>= IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA1_THRESHOLD_NUMBER_SHIFT;
+
+              snprintf (tmpbuf,
+                        tmpbuflen,
+                        "Threshold Exceeded, Threshold Number = %u",
+                        threshold_number);
+
+              return (1);
+              
+            }
+        }
     }
 
  out:
@@ -645,6 +660,12 @@ sel_string_output_intel_node_manager_event_data2_class_oem (ipmi_sel_ctx_t ctx,
           
 	  switch (error_type)
 	    {
+	    case IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_OUTLET_TEMPERATURE_READING_FAILURE:
+ 	      error_type_str = "Outlet Temperature Reading Failure";
+	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_VOLUMETRIC_AIRFLOW_READING_FAILURE:
+ 	      error_type_str = "Volumetric Airflow Reading Failure";
+	      break;
 	    case IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_POLICY_MISCONFIGURATION:
 	      error_type_str = "Policy Misconfiguration";
 	      break;
@@ -660,6 +681,9 @@ sel_string_output_intel_node_manager_event_data2_class_oem (ipmi_sel_ctx_t ctx,
 	    case IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_REAL_TIME_CLOCK_SYNCHRONIZATION_FAILURE:
 	      error_type_str = "Real-time clock synchronization failure";
 	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_PLATFORM_SHUTDOWN_INITIATED_BY_INTEL_NM_POLICY:
+	      error_type_str = "Platform shutdown initiated by Intel NM policy";
+	      break;
 	    default:
 	      error_type_str = "Unknown";
 	    }
@@ -745,6 +769,18 @@ sel_string_output_intel_node_manager_event_data2_class_oem (ipmi_sel_ctx_t ctx,
 	    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_FIRMWARE_EXCEPTION:
 	      health_event_str = "Firmware Exception";
 	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_FLASH_WEAR_OUT_PROTECTION_WARNING:
+	      health_event_str = "Flash Wear-Out Protection Warning";
+	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_PECI_OVER_DMI_INTERFACE_ERROR:
+	      health_event_str = "PECI over DMI interface error";
+	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_MCTP_INTERFACE_ERROR:
+	      health_event_str = "MCTP interface error";
+	      break;
+	    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_AUTO_CONFIGURATION_FINISHED:
+	      health_event_str = "Auto-configuration finished";
+	      break;
 	    default:
 	      health_event_str = "Unknown";
 	    }
@@ -756,12 +792,81 @@ sel_string_output_intel_node_manager_event_data2_class_oem (ipmi_sel_ctx_t ctx,
           
           return (1);
         }
+
+      if (system_event_record_data->event_type_code == IPMI_EVENT_READING_TYPE_CODE_OEM_INTEL_NODE_MANAGER_CUPS_EVENT
+	  && system_event_record_data->sensor_number == IPMI_SENSOR_NUMBER_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_SENSOR)
+        {
+          uint8_t domain_id;
+	  char domain_id_str[INTEL_NODE_MANAGER_EVENT_BUFFER_LENGTH + 1];
+          
+          domain_id = (system_event_record_data->event_data2 & IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA2_DOMAIN_ID_BITMASK);
+          domain_id >>= IPMI_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_EVENT_DATA2_DOMAIN_ID_SHIFT;
+          
+	  memset (domain_id_str, '\0', INTEL_NODE_MANAGER_EVENT_BUFFER_LENGTH + 1);
+
+	  _sel_string_output_intel_node_manager_domain_id (ctx,
+							   domain_id_str,
+							   INTEL_NODE_MANAGER_EVENT_BUFFER_LENGTH,
+							   domain_id);
+
+          snprintf (tmpbuf,
+                    tmpbuflen,
+                    "Domain ID = %s",
+                    domain_id_str);
+
+          return (1);
+        }
     }
 
  out:
   return (0);
 }
 
+/* return (0) - no OEM match
+ * return (1) - OEM match
+ * return (-1) - error, cleanup and return error
+ */
+int
+sel_string_output_intel_node_manager_event_data3_discrete_oem (ipmi_sel_ctx_t ctx,
+							       struct ipmi_sel_entry *sel_entry,
+							       uint8_t sel_record_type,
+							       char *tmpbuf,
+							       unsigned int tmpbuflen,
+							       unsigned int flags,
+							       unsigned int *wlen,
+							       struct ipmi_sel_system_event_record_data *system_event_record_data)
+{
+  assert (ctx);
+  assert (ctx->magic == IPMI_SEL_CTX_MAGIC);
+  assert (sel_entry);
+  assert (tmpbuf);
+  assert (tmpbuflen);
+  assert (!(flags & ~IPMI_SEL_STRING_FLAGS_MASK));
+  assert (flags & IPMI_SEL_STRING_FLAGS_INTERPRET_OEM_DATA);
+  assert (wlen);
+  assert (system_event_record_data);
+
+  if (system_event_record_data->sensor_type == IPMI_SENSOR_TYPE_OEM_INTEL_NODE_MANAGER
+      && system_event_record_data->event_type_code == IPMI_EVENT_READING_TYPE_CODE_STATE
+      && system_event_record_data->sensor_number == IPMI_SENSOR_NUMBER_OEM_INTEL_NODE_MANAGER_NM_SMART_CLST_SENSOR
+      && system_event_record_data->event_data3_flag == IPMI_SEL_EVENT_DATA_OEM_CODE)
+    {
+      if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_NM_SMART_CLST_SENSOR_EVENT_DATA3_EXTERNAL)
+	snprintf (tmpbuf,
+		  tmpbuflen,
+		  "Source of assertion is external");
+      else
+	snprintf (tmpbuf,
+		  tmpbuflen,
+		  "Power Supply Status Number = %u",
+		  system_event_record_data->event_data3);
+      
+      return (1);
+    }
+
+  return (0);
+}
+
 /* return (0) - no OEM match
  * return (1) - OEM match
  * return (-1) - error, cleanup and return error
@@ -822,7 +927,8 @@ sel_string_output_intel_node_manager_event_data3_class_oem (ipmi_sel_ctx_t ctx,
           error_type = (system_event_record_data->event_data2 & IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_BITMASK);
           error_type >>= IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_SHIFT;
           
-          if (error_type == IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_POLICY_MISCONFIGURATION)
+          if (error_type == IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_POLICY_MISCONFIGURATION
+	      || error_type == IPMI_OEM_INTEL_NODE_MANAGER_HEALTH_EVENT_EVENT_DATA2_ERROR_TYPE_PLATFORM_SHUTDOWN_INITIATED_BY_INTEL_NM_POLICY)
             {
               snprintf (tmpbuf,
                         tmpbuflen,
@@ -872,7 +978,7 @@ sel_string_output_intel_node_manager_event_data3_class_oem (ipmi_sel_ctx_t ctx,
 		{
 		  snprintf (tmpbuf,
 			    tmpbuflen,
-			    "recovery bootloader image or factory presets image corrupted");
+			    "flash partition table, recovery image or factory presets image corrupted");
 		  
 		  return (1);
 		}
@@ -946,12 +1052,215 @@ sel_string_output_intel_node_manager_event_data3_class_oem (ipmi_sel_ctx_t ctx,
 	    }
 	  else if (system_event_record_data->event_data2 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_MANUFACTURING_ERROR)
 	    {
-	      if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_INTEL_ME_FW_CONFIGURATION_BAD)
+	      if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_GENERIC_ERROR)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Generic error");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_WRONG_OR_MISSING_VSCC_TABLE)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Wrong or missing VSCC table");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_WRONG_SENSOR_SCANNING_PERIOD_IN_PIA)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Wrong sensor scanning period in PIA");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_WRONG_DEVICE_DEFINITION_IN_PIA)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Wrong device definition in PIA");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_WRONG_SMART_CLST_CONFIGURATION)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Wrong SMART/CLST configuration");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_INTEL_ME_FW_CONFIGURATION_IS_INCONSISTENT_OR_OUT_OF_RANGE)
 		{
 		  snprintf (tmpbuf,
 			    tmpbuflen,
 			    "Intel ME FW configuration is inconsistent or out of range");
 		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MANUFACTURING_ERROR_PERCENTAGE_OF_FLASH_WRITE_OPERATIONS_WHICH_HAVE_BEEN_CONDUCTED)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "percentage of flash write operations which have been conducted");
+		  
+		  return (1);
+		}
+	    }
+	  else if (system_event_record_data->event_data2 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_PECI_OVER_DMI_INTERFACE_ERROR)
+	    {
+	      if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_DRAM_INIT_DONE_EVENT_NOT_RECEIVED)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "DRAM Init Done event not received");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_MCTP_SAD_REGISTER_NOT_CORRECTLY_CONFIGURED_BY_BIOS)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "MCTP SAD Register not correctly configured by BIOS");
+		  
+		  return (1);
+		}
+	      else if (system_event_record_data->event_data3 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_DMI_TIMEOUT_OF_PECI_REQUEST)
+		{
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "DMI timeout of PECI request");
+		  
+		  return (1);
+		}
+	    }
+	  else if (system_event_record_data->event_data2 == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA2_AUTO_CONFIGURATION_FINISHED)
+	    {
+	      uint8_t auto_configuration_finished_result;
+
+	      auto_configuration_finished_result = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_BITMASK);
+	      auto_configuration_finished_result >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SHIFT;
+
+	      if (auto_configuration_finished_result == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS)
+		{
+		  uint8_t dc_power_source, chassis_power_input_source, psu_efficiency_source, unmanaged_power_source;
+		  char *dc_power_source_str, *chassis_power_input_source_str, *psu_efficiency_source_str, *unmanaged_power_source_str;
+
+		  dc_power_source = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_DC_POWER_SOURCE_BITMASK);
+		  dc_power_source >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_DC_POWER_SOURCE_SHIFT;
+
+		  switch (dc_power_source)
+		    {
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_DC_POWER_SOURCE_BMC:
+		      dc_power_source_str = "BMC";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_DC_POWER_SOURCE_PSU:
+		      dc_power_source_str = "PSU";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_DC_POWER_SOURCE_ON_BOARD_POWER_SENSOR:
+		      dc_power_source_str = "On-board power sensor";
+		      break;
+		    default:
+		      dc_power_source_str = "Unknown";
+		    }
+
+		  chassis_power_input_source = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_BITMASK);
+		  chassis_power_input_source >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_SHIFT;
+
+		  switch (chassis_power_input_source)
+		    {
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_BMC:
+		      chassis_power_input_source_str = "BMC";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_PSU:
+		      chassis_power_input_source_str = "PSU";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_ON_BOARD_POWER_SENSOR_PSU_EFFICIENCY:
+		      chassis_power_input_source_str = "On-board power sensor / PSU efficiency";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_CHASSIS_POWER_INPUT_SOURCE_NOT_SUPPORTED:
+		      chassis_power_input_source_str = "not supported";
+		      break;
+		    default:
+		      chassis_power_input_source_str = "Unknown";
+		    }
+
+		  psu_efficiency_source = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_PSU_EFFICIENCY_SOURCE_BITMASK);
+		  psu_efficiency_source >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_PSU_EFFICIENCY_SOURCE_SHIFT;
+
+		  switch (psu_efficiency_source)
+		    {
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_PSU_EFFICIENCY_SOURCE_BMC:
+		      psu_efficiency_source_str = "BMC";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_PSU_EFFICIENCY_SOURCE_PSU:
+		      psu_efficiency_source_str = "PSU";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_PSU_EFFICIENCY_SOURCE_NOT_SUPPORTED:
+		      psu_efficiency_source_str = "not supported";
+		      break;
+		    default:
+		      psu_efficiency_source_str = "Unknown";
+		    }
+
+		  unmanaged_power_source = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_UNMANAGED_POWER_SOURCE_BITMASK);
+		  unmanaged_power_source >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_UNMANAGED_POWER_SOURCE_SHIFT;
+
+		  switch (unmanaged_power_source)
+		    {
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_UNMANAGED_POWER_SOURCE_BMC:
+		      unmanaged_power_source_str = "BMC";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_SUCCESS_UNMANAGED_POWER_SOURCE_ESTIMATED:
+		      unmanaged_power_source_str = "estimated";
+		      break;
+		    default:
+		      unmanaged_power_source_str = "Unknown";
+		    }
+
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Auto Configuration Success - DC Power Source = %s, Chassis Power input source = %s, PSU efficiency source = %s, Unmanaged power source = %s",
+			    dc_power_source_str,
+			    chassis_power_input_source_str,
+			    psu_efficiency_source_str,
+			    unmanaged_power_source_str);
+		  
+		  return (1);
+		}
+	      else /* auto_configuration_finished_result == IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE */
+		{
+		  uint8_t failure;
+		  char *failure_str;
+
+		  failure = (system_event_record_data->event_data3 & IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_BITMASK);
+		  failure >>= IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_SHIFT;
+
+		  switch (failure)
+		    {
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_BMC_DISCOVERY_FAILURE:
+		      failure_str = "BMC discovery failure";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_INSUFFICIENT_FACTORY_CONFIGURATION:
+		      failure_str = "Insufficient factory configuration";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_UNKNOWN_SENSOR_TYPE:
+		      failure_str = "Unknown sensor type";
+		      break;
+		    case IPMI_OEM_INTEL_NODE_MANAGER_INTEL_ME_FIRMWARE_HEALTH_EVENT_EVENT_DATA3_AUTO_CONFIGURATION_FINISHED_RESULT_FAILURE_FAILURE_OTHER_ERROR_ENCOUNTERED:
+		      failure_str = "Other error encountered";
+		      break;
+		    default:
+		      failure_str = "Unknown";
+		    }
+
+		  snprintf (tmpbuf,
+			    tmpbuflen,
+			    "Auto Configuration Failure - %s",
+			    failure_str);
+		  
 		  return (1);
 		}
 	    }
@@ -963,6 +1272,17 @@ sel_string_output_intel_node_manager_event_data3_class_oem (ipmi_sel_ctx_t ctx,
 	  
           return (1);
         }
+
+      if (system_event_record_data->event_type_code == IPMI_EVENT_READING_TYPE_CODE_OEM_INTEL_NODE_MANAGER_CUPS_EVENT
+	  && system_event_record_data->sensor_number == IPMI_SENSOR_NUMBER_OEM_INTEL_NODE_MANAGER_CUPS_EVENT_SENSOR)
+        {
+          snprintf (tmpbuf,
+                    tmpbuflen,
+                    "Policy ID = %u",
+                    system_event_record_data->event_data3);
+          
+          return (1);
+	}
     }
   
  out:
