@@ -33,8 +33,8 @@ int ipmiconsole_ctx_setup (ipmiconsole_ctx_t c);
 
 void ipmiconsole_ctx_cleanup (ipmiconsole_ctx_t c);
 
-/* Wrapper for list callback */
-void ipmiconsole_ctx_list_cleanup (ipmiconsole_ctx_t c);
+/* Wrapper for list callback on console_engine_ctxs_to_destroy */
+void ipmiconsole_ctx_garbage_collection_cleanup (ipmiconsole_ctx_t c);
 
 int ipmiconsole_ctx_config_setup (ipmiconsole_ctx_t c,
                                   const char *hostname,
